@@ -2,10 +2,13 @@
  * Common definitions and declarations for the wlib library
  */
 
-#ifndef WIN_H
-#define WIN_H
+#ifndef HAVE_WLIB_H
+#define HAVE_WLIB_H
 #ifdef WINDOWS
 #include <stdio.h>
+#define FILE_SEP_CHAR "\\"
+#else
+#define FILE_SEP_CHAR "/"
 #endif
 
 #ifdef USE_SIMPLE_GETTEXT
@@ -51,6 +54,7 @@ typedef struct wIcon_t      * wIcon_p;
 typedef struct wDrawBitMap_t * wDrawBitMap_p;
 typedef struct wFont_t      * wFont_p;
 typedef struct wBitmap_t	* wBitmap_p;
+typedef struct wStatus_t    * wStatus_p;
 typedef int wDrawWidth;
 typedef int wDrawColor;
 
@@ -62,6 +66,70 @@ typedef struct {
 extern long debugWindow;
 extern long wDebugFont;
 
+/*------------------------------------------------------------------------------
+ *
+ * Bitmap Controls bitmap.c
+ */
+
+wControl_p wBitmapCreate(wWin_p parent, wPos_t x, wPos_t y, long options, wIcon_p iconP);
+wIcon_p wIconCreateBitMap(wPos_t w, wPos_t h, const char *bits, wDrawColor color);
+wIcon_p wIconCreatePixMap(char *pm[]);
+void wIconSetColor(wIcon_p ip, wDrawColor color);
+
+/*------------------------------------------------------------------------------
+ *
+ * Frames around widgets boxes.c
+ *
+ */
+
+typedef enum {
+	wBoxThinB,
+	wBoxThinW,
+	wBoxAbove,
+	wBoxBelow,
+	wBoxThickB,
+	wBoxThickW,
+	wBoxRidge,
+	wBoxTrough }
+		wBoxType_e;
+
+void wBoxSetSize(wBox_p b, wPos_t w, wPos_t h);
+void wlibDrawBox(wWin_p win, wBoxType_e style, wPos_t x, wPos_t y, wPos_t w, wPos_t h);
+wBox_p wBoxCreate(wWin_p parent, wPos_t bx, wPos_t by, const char *labelStr, wBoxType_e boxTyp, wPos_t bw, wPos_t bh);
+
+/*------------------------------------------------------------------------------
+ *
+ * Buttons, toggles and radiobuttons button.c
+ *
+ */
+
+/* Creation CallBacks */
+typedef void (*wButtonCallBack_p)( void * );
+
+/* Creation Options */
+#define BB_DEFAULT	(1L<<5)
+#define BB_CANCEL	(1L<<6)
+#define BB_HELP (1L<<7)
+
+/* Creation CallBacks */
+typedef void (*wChoiceCallBack_p)( long, void * );
+
+/* Creation Options */
+#define BC_ICON 	(1L<<0)
+#define BC_HORZ 	(1L<<22)
+#define BC_NONE 	(1L<<19)
+#define BC_NOBORDER 	(1L<<15)
+
+void wButtonSetLabel(wButton_p bb, const char *labelStr);
+void wButtonSetBusy(wButton_p bb, int value);
+wButton_p wButtonCreate(wWin_p parent, wPos_t x, wPos_t y, const char *helpStr, const char *labelStr, long option, wPos_t width, wButtonCallBack_p action, void *data);
+void wRadioSetValue(wChoice_p bc, long value);
+long wRadioGetValue(wChoice_p bc);
+void wToggleSetValue(wChoice_p bc, long value);
+long wToggleGetValue(wChoice_p b);
+wChoice_p wRadioCreate(wWin_p parent, wPos_t x, wPos_t y, const char *helpStr, const char *labelStr, long option, const char **labels, long *valueP, wChoiceCallBack_p action, void *data);
+wChoice_p wToggleCreate(wWin_p parent, wPos_t x, wPos_t y, const char *helpStr, const char *labelStr, long option, const char **labels, long *valueP, wChoiceCallBack_p action, void *data);
+
 
 /*------------------------------------------------------------------------------
  *
@@ -137,10 +205,11 @@ FILE * wFileOpen(		const char *, const char * );
 typedef enum {
 	wClose_e,
 	wResize_e,
+	wState_e,
 	wQuit_e,
 	wRedraw_e }
 		winProcEvent;
-typedef void (*wWinCallBack_p)( wWin_p, winProcEvent, void * );
+typedef void (*wWinCallBack_p)( wWin_p, winProcEvent, void *, void * );
 
 /* Creation Options */
 #define F_AUTOSIZE	(1L<<1)
@@ -153,7 +222,8 @@ typedef void (*wWinCallBack_p)( wWin_p, winProcEvent, void * );
 #define F_RECALLSIZE	(1L<<10)
 #define F_TOP		(1L<<11)
 #define F_CENTER	(1L<<12)
-#define F_HIDE		(1L<<12)
+#define F_HIDE		(1L<<13)
+#define F_MAXIMIZE  (1L<<14)
 
 wWin_p wWinMainCreate(	        const char *, wPos_t, wPos_t, const char *, const char *, const char *,
 				long, wWinCallBack_p, void * );
@@ -165,6 +235,7 @@ void wWinSetBigIcon(		wWin_p, wIcon_p );
 void wWinSetSmallIcon(		wWin_p, wIcon_p );
 void wWinShow(			wWin_p, wBool_t );
 wBool_t wWinIsVisible(		wWin_p );
+wBool_t wWinIsMaximized( wWin_p win);
 void wWinGetSize (		wWin_p, wPos_t *, wPos_t * );
 void wWinSetSize(		wWin_p, wPos_t, wPos_t );
 void wWinSetTitle(		wWin_p, const char * );
@@ -212,50 +283,6 @@ void wControlHilite(		wControl_p, wBool_t );
 void wControlLinkedSet( wControl_p b1, wControl_p b2 );
 void wControlLinkedActive( wControl_p b, int active );
 
-/*------------------------------------------------------------------------------
- *
- * Push buttons
- */
-
-/* Creation CallBacks */
-typedef void (*wButtonCallBack_p)( void * );
-
-/* Creation Options */
-#define BB_DEFAULT	(1L<<5)
-#define BB_CANCEL	(1L<<6)
-#define BB_HELP (1L<<7)
-
-wButton_p wButtonCreate(	wWin_p, wPos_t, wPos_t, const char *, const char *, long,
-				wPos_t, wButtonCallBack_p, void * );
-void wButtonSetLabel(		wButton_p, const char * );
-void wButtonSetColor(		wButton_p, wDrawColor );
-void wButtonSetBusy(		wButton_p, wBool_t );
-
-
-/*------------------------------------------------------------------------------
- *
- * Radio and Toggle (Choice) Buttons
- */
-
-/* Creation CallBacks */
-typedef void (*wChoiceCallBack_p)( long, void * );
-
-/* Creation Options */
-#define BC_ICON 	(1L<<0)
-#define BC_HORZ 	(1L<<22)
-#define BC_NONE 	(1L<<19)
-#define BC_NOBORDER 	(1L<<15)
-
-wChoice_p wRadioCreate(		wWin_p, wPos_t, wPos_t, const char *, const char *, long,
-				const char **, long *, wChoiceCallBack_p, void * );
-wChoice_p wToggleCreate(	wWin_p, wPos_t, wPos_t, const char *, const char *, long,
-				const char **, long *, wChoiceCallBack_p, void * );
-void wRadioSetValue(		wChoice_p, long );
-void wToggleSetValue(		wChoice_p, long );
-long wRadioGetValue(		wChoice_p );
-long wToggleGetValue(		wChoice_p );
-
-
 /*------------------------------------------------------------------------------
  *
  * String entry
@@ -311,29 +338,35 @@ typedef void (*wListCallBack_p)( wIndex_t, const char *, wIndex_t, void *, void
 #define BL_EDITABLE	(1L<<23)
 #define BL_ICON		(1L<<0)
 
+
+/* lists, droplists and combo boxes */
+
 wList_p wListCreate(		wWin_p, wPos_t, wPos_t, const char *, const char *, long,
 				long, wPos_t, int, wPos_t *, wBool_t *, const char **, long *, wListCallBack_p, void * );
-wList_p wComboListCreate(	wWin_p, wPos_t, wPos_t, const char *, const char *, long,
-				long, wPos_t, long *, wListCallBack_p, void * );
 wList_p wDropListCreate(	wWin_p, wPos_t, wPos_t, const char *, const char *, long,
 				long, wPos_t, long *, wListCallBack_p, void * );
-void wListClear(		wList_p );
-void wListSetIndex(		wList_p, wIndex_t );
+				
+wList_p wComboListCreate(wWin_p parent, wPos_t x, wPos_t y, const char *helpStr, const char *labelStr, long option, long number, wPos_t width, long *valueP, wListCallBack_p action, void *data);	
+void wListClear(wList_p b);
+void wListSetIndex(wList_p b, int element);
+wIndex_t wListFindValue(wList_p b, const char *val);
+wIndex_t wListGetCount(wList_p b);
 wIndex_t wListGetIndex(		wList_p );
-wIndex_t wListFindValue(	wList_p, const char * );
+void *wListGetItemContext(wList_p b, wIndex_t inx);
+wBool_t wListGetItemSelected(wList_p b, wIndex_t inx);
+wIndex_t wListGetSelectedCount(wList_p b);
+void wListSelectAll(wList_p bl);
+wBool_t wListSetValues(wList_p b, wIndex_t row, const char *labelStr, wIcon_p bm, void *itemData);
+void wListDelete(wList_p b, wIndex_t inx);
+int wListGetColumnWidths(wList_p bl, int colCnt, wPos_t *colWidths);
+wIndex_t wListAddValue(wList_p b, const char *labelStr, wIcon_p bm, void *itemData);
+void wListSetSize(wList_p bl, wPos_t w, wPos_t h);
+wIndex_t wListGetValues(	wList_p, char *, int, void * *, void * * );
+
+/** \todo Check for the existance of following functions */
 void  wListSetValue(		wList_p, const char * );
-int wListGetColumnWidths(	wList_p, int, wPos_t * );
-wBool_t wListSetValues(		wList_p, wIndex_t, const char *, wIcon_p, void * );
 void wListSetActive(		wList_p, wIndex_t, wBool_t );
 void wListSetEditable(		wList_p, wBool_t );
-wIndex_t wListAddValue(		wList_p, const char *, wIcon_p, void * );
-void wListDelete(		wList_p, wIndex_t );
-wIndex_t wListGetValues(	wList_p, char *, int, void * *, void * * );
-wIndex_t wListGetCount(		wList_p );
-void * wListGetItemContext(	wList_p, wIndex_t );
-wBool_t wListGetItemSelected(	wList_p, wIndex_t );
-wIndex_t wListGetSelectedCount(	wList_p );
-void wListSetSize(		wList_p, wPos_t, wPos_t );
 
 
 /*------------------------------------------------------------------------------
@@ -343,6 +376,7 @@ void wListSetSize(		wList_p, wPos_t, wPos_t );
 
 #define BM_LARGE (1L<<24)
 #define BM_SMALL (1L<<25)
+#define COMBOBOX (1L)
 
 #define wMessageSetFont( x ) ( x & (BM_LARGE | BM_SMALL ))
 
@@ -352,29 +386,10 @@ wMessage_p wMessageCreateEx(	wWin_p, wPos_t, wPos_t, const char *,
 
 void wMessageSetValue(		wMessage_p, const char * );
 void wMessageSetWidth(		wMessage_p, wPos_t );
+wPos_t wMessageGetWidth( const char *testString );
 wPos_t wMessageGetHeight( long );
 
 
-/*------------------------------------------------------------------------------
- *
- * Boxes
- */
-
-typedef enum {
-	wBoxThinB,
-	wBoxThinW,
-	wBoxAbove,
-	wBoxBelow,
-	wBoxThickB,
-	wBoxThickW,
-	wBoxRidge,
-	wBoxTrough }
-		wBoxType_e;
-wBox_p wBoxCreate(		wWin_p, wPos_t, wPos_t, const char *, wBoxType_e,
-				wPos_t, wPos_t );
-void wBoxSetSize(		wBox_p, wPos_t, wPos_t );
-
-
 /*------------------------------------------------------------------------------
  *
  * Lines
@@ -457,7 +472,8 @@ typedef void (*wDrawActionCallBack_p)(	wDraw_p, void*, wAction_t, wPos_t, wPos_t
 /* Creation Options */
 #define BD_TICKS	(1L<<25)
 #define BD_DIRECT	(1L<<26)
-#define BD_NOCAPTURE	(1L<<27)
+#define BD_NOCAPTURE (1L<<27)
+#define BD_NOFOCUS  (1L<<28)
 
 /* Create: */
 wDraw_p wDrawCreate(		wWin_p, wPos_t, wPos_t, const char *, long,
@@ -497,7 +513,7 @@ long wDrawGetRGB(		wDrawColor );
 /* Geometry */
 double wDrawGetDPI(		wDraw_p );
 double wDrawGetMaxRadius(	wDraw_p );
-void wDrawSetSize(		wDraw_p, wPos_t, wPos_t );
+void wDrawSetSize(		wDraw_p, wPos_t, wPos_t, void * );
 void wDrawGetSize(		wDraw_p, wPos_t *, wPos_t * );
 
 /* Bitmaps */
@@ -585,7 +601,10 @@ typedef enum {
 	wAccelKey_F9,
 	wAccelKey_F10,
 	wAccelKey_F11,
-	wAccelKey_F12 }
+	wAccelKey_F12,
+    wAccelKey_Numpad_Add,
+    wAccelKey_Numpad_Subtract,
+	wAccelKey_LineFeed }
 	wAccelKey_e;
 
 /* Creation CallBacks */
@@ -636,13 +655,15 @@ void wAttachAccelKey( wAccelKey_e, int, wAccelKeyCallBack_p, void * );
  * File Selection
  */
 
+#define FS_MULTIPLEFILES	1
+
 struct wFilSel_t;
 typedef enum {
 	FS_SAVE,
 	FS_LOAD,
 	FS_UPDATE }
 		wFilSelMode_e;
-typedef int (*wFilSelCallBack_p)( const char * pathName, const char * fileName, void * );
+typedef int (*wFilSelCallBack_p)( int files, char ** fileName, void * );
 struct wFilSel_t * wFilSelCreate(wWin_p, wFilSelMode_e, int, const char *, const char *,
 				wFilSelCallBack_p, void * );
 int wFilSelect(			struct wFilSel_t *, const char * );
@@ -666,12 +687,21 @@ wDrawColor wColorSelectButtonGetColor( wButton_p );
  * Preferences
  */
 
-void wPrefSetString(		const char *, const char *, const char * );
-const char * wPrefGetString(		const char *, const char * );
-void wPrefSetInteger(		const char *, const char *, long );
-wBool_t wPrefGetInteger(	const char *, const char *, long *, long );
+void wPrefSetString(const char *, const char *, const char * );
+char * wPrefGetString(const char *section, const char *name);
+char * wPrefGetStringBasic( const char *section, const char *name );
+char * wPrefGetStringExt(const char *section, const char *name);
+
+void wPrefSetInteger(const char *, const char *, long );
+wBool_t wPrefGetInteger(const char *section, const char *name, long *result, long defaultValue);
+wBool_t wPrefGetIntegerBasic(const char *section, const char *name, long *result, long defaultValue);
+wBool_t wPrefGetIntegerExt(const char *section, const char *name, long *result, long defaultValue);
+
 void wPrefSetFloat(		const char *, const char *, double );
-wBool_t wPrefGetFloat(		const char *, const char *, double *, double );
+wBool_t wPrefGetFloat(const char *section, const char *name, double *result, double defaultValue);
+wBool_t wPrefGetFloatBasic(const char *section, const char *name, double *result, double defaultValue);
+wBool_t wPrefGetFloatExt(const char *section, const char *name, double *result, double defaultValue);
+
 const char * wPrefGetSectionItem( const char * sectionName, wIndex_t * index, const char ** name );
 void wPrefFlush(		void );
 void wPrefReset(		void );
@@ -680,9 +710,20 @@ void CleanupCustom( void );
 
 /*------------------------------------------------------------------------------
  *
- * Bitmap Controls
+ * Statusbar
  */
 
-wControl_p wBitmapCreate( wWin_p parent, wPos_t xx, wPos_t yy, long options, wIcon_p iconP );
+wStatus_p wStatusCreate(
+    wWin_p	parent,
+    wPos_t	x,
+    wPos_t	y,
+    const char 	* labelStr,
+    wPos_t	width,
+    const char	*message );
 
+wPos_t wStatusGetWidth(const char *testString);
+wPos_t wStatusGetHeight(long flags);
+
+void wStatusSetValue(wStatus_p b, const char * arg);
+void wStatusSetWidth(wStatus_p b, wPos_t width);
 #endif
